.module       clicko;

.include <system.k>;
.include <midi.k>;

.global         handle_MIDI_controller;
.external       process_midi;
.external       rx_buf,tx_buf;
.global         main_loop,input_samples,init_params;
.global irqeisr;
.external getrand;

{ signal processing variables here }


{ internal variables }
.var wetmix,drymix;
.init wetmix:0x7fff;
.init drymix:0;

.var dryinput;

.var decayrate;
.init decayrate:0x7f00;

.var outval;
.init outval:0;

.var lastinput;
.init lastinput:0;

.var lastoutput;
.init lastoutput:0;

{ parameters here }

{ do any dynamic memory/variable initializing you need to here }
init_params:
        nop;
        nop;
        nop;
        rts;

{ command loop - check for incoming data on serial port }
main_loop:
        ar = dm (CHAR_WAITING_FLAG);    { check serial port for MIDI data }
        none = pass ar;
        if ne jump main_loop;
        call process_midi;        { if MIDI byte received }
        jump main_loop;




{ SPORT0 interrupt handler -- this is called at 44.1KHz }
input_samples:
        ena sec_reg;                { use shadow register bank }

        { call getrand; } { keep random number thing spinning }

        mx0 = dm(rx_buf +1);    { read a/d converters }
        mx1 = dm(rx_buf +2);    { leave left/right values in my0,my1 }

        { combine inputs into one through averaging }              
        my1 = 0x3fff;
        mr  =      mx0*my1(ss);
        mr  = mr + mx1*my1(ss);
        ar  = mr1;
        ax0 = ar;       
        dm(dryinput)=ax0;

        { did a sign change occur? }
        ay1 = dm(lastinput);
        dm(lastinput)=ax0;      { save for next time }
        ar  = ar xor ay1;

        ax1 = ar;
        ar  = tstbit 15 of ar;
        ar  = ax1;
        if eq jump nosignchange;

        set fl1;

        { yes, sign change occurred. }
        { is it no positive or negative? }

        ar  =  ax0 - ay1;  { this one - last one }
        ay1 = ar;
        dm(outval)=ay1;
        jump burtwilson;

nosignchange:
        reset fl1;

        { ay0 = 0; }  { for simple noise heh }
        mx0 = dm(outval);
        my0 = dm(decayrate);
        {my0 = 0x7f80;  } { decay rate }
        mr  = mx0*my0(ss);
        dm(outval)=mr1;
        ay1 = mr1;
        
        jump burtwilson;

burtwilson:
        { okay, so the wet is in ay1. let's mix the dry in }
        mx0 = dm(wetmix);
        my0 = ay1;
        mx1 = dm(drymix);
        my1 = dm(dryinput);
        mr  = mx0*my0(ss);
        mr  = mr + mx1*my1(ss);  { shouldn't overflow }

        dm(tx_buf+ 1)=mr1;      { write d/a converters }
        dm(tx_buf+ 2)=mr1;      { write left/right values from my0, my1 }
        
thatwasfun:
        rti;    





{ Here is where MIDI messages are used to set variables in your Effect! }
handle_MIDI_controller:
        ax1 = dm(midi_cont);
        ar  = dm(midi_val);
        ay1 = 1;none = ax1-ay1;if eq jump set_blah1;
        ay1 = 127;none = ax1-ay1;if eq jump set_blah127;
        rts;

set_blah1:
        ay1 = dm(midi_val);
        rts;

set_blah127:
        ay1 = dm(midi_val);
        rts;

{ when user presses the white interrupt button }
irqeisr:
        ena sec_reg;

        call getrand;   { returns numb in ax0 }
        ay0 = 0x03ff;
        ar  = ax0 and ay0;
        ay1 = 0x7c00;
        ar  = ar  or ay1;
        dm(decayrate)=ar;       { 7f00 <=> 7fff }

        call getrand;
        ay0 = 0x7fff;
        ar  = ay0 - ax0;
        dm(wetmix)=ax0;
        dm(drymix)=ar;

        rti;

.endmod;
